package Net::Ajp13::Container::CGI;
use base 'Net::Ajp13::Container';
use strict;

sub invoke {
    my $self = shift;
    my $request = shift;
    my $response = shift;
    my $message_handler = shift;

    # invoke CGI handler
    eval {
        my $class = $request->uri;
        my $mount = $message_handler->getconf('MountPath');
        my $path = $class;
        while ($path ne '/') {
            $path =~ s|/[^/]*/?$|/|;
            next unless exists $mount->{$path};
            $class =~ s|^$path|$mount->{$path}|;
            last;
        }

        my $class_path = $class;
        die '404' unless -r $class_path;
        local %ENV = $self->get_cgi_env($request);;
        local *STDIN;
        local *STDOUT;
        local *STDERR;
        tie *STDOUT, 'Net::Ajp13::Container::CGI::IO', $request, $response;

        $response->status(200);
        do "$class_path";


    };
    if ($@) {
        if ($@ =~ /^404/) {
            $response->status(404);
        }
        else {
            $response->status(500);
            return $self->send_error($request, $response, $self->_stacktrace());
        }
    }

    if ($response->status >= 300) {
        return $self->send_error($request, $response);
    }

    return $response->close();
}

sub get_cgi_env {
    my $self = shift;
    my $req = shift;
    my %env = ();

    $env{SCRIPT_FILENAME} = $req->uri;
    $env{QUERY_STRING} = $req->query_string;
    $env{PATH_INFO} = $req->path_info;
    for my $name (keys %{$req->{header}}) {
        my $cgi_name = 'HTTP_'. uc $name;
        $env{$cgi_name} = $req->header($name);
    }

    return %env;
}

package Net::Ajp13::Container::CGI::IO;
use base 'Tie::Handle';

sub TIEHANDLE {
    my $class = shift;
    my ($req, $res) = @_;
    bless {
        _request => $req,
        _response => $res,
        header => 0,
    }, $class;
}

sub WRITE {
    my $self = shift;
    my ($buf, $len, $offset) = @_;
    $self->{_response}->print(substr $buf, $offset, $len);
}

sub PRINT {
    my $self = shift;
    if ($self->{header} == 0) {
        my $str = join $,, @_;
        while ($str =~ m/([^:]+):\s+([^\r\n]+)\r?\n(\r?\n(.*)?)?/g) {
            my ($k, $v, $end, $body) = ($1, $2, $3, $4);
            if (lc $k eq 'content-type') {
                $self->{_response}->content_type($v);
            }
            $self->{_response}->header($k, $v);
            if ($end) {
                $self->{header} = 1;
                $self->{_response}->print($body);
                last;
            }
        }
        return;
    }

    $self->{_response}->print(join($,, @_));
}

sub PRINTF {
    my $self = shift;
    my $fmt = shift;
    my $str = sprintf $fmt, @_;

    $self->PRINT($str);
}

1;
__END__
