package Net::Ajp13::Container::ModPerl;
use base 'Net::Ajp13::Container';

use Apache::Constants qw(OK);
use strict;


sub invoke {
    my $self = shift;
    my $request = shift;
    my $response = shift;
    my $message_handler = shift;

    # invoke mod_perl handler
    eval {
        my $class = $request->uri;
        my $mount = $message_handler->getconf('MountPath');
        my $path = $class;
        while ($path ne '/') {
            $path =~ s|/[^/]*/?$|/|;
            next unless exists $mount->{$path};
            $class =~ s|^$path|$mount->{$path}|;
            last;
        }

        my $class_path = sprintf '%s.pm', $class;
        die '404' unless -r $class_path;
        push @INC, $mount->{$path};
        require $class_path;

        my $classname = $self->{classes}->{$class_path};
        my $func = eval "\\&{${classname}::handler}";
        die $@ if $@;
        my $r = $self->create_modperl_env($request, $response, $message_handler);
        my $rc = $func->($r); 
        if ($rc != OK) {
            $response->status($rc);
        }
    };
    if ($@) {
        if ($@ =~ /^404/) {
            $response->status(404);
        }
        else {
            $response->status(500);
            return $self->send_error($request, $response, $self->_stacktrace());
        }
    }

    if ($response->status >= 300) {
        return $self->send_error($request, $response);
    }

    return $response->close();
}

sub create_modperl_env {
    my $self = shift;
    my $req = shift;
    my $res = shift;
    my $mhandler = shift;

    my $r = Net::Ajp13::Container::ModPerl::Apache->new($self, $req, $res, $mhandler);

    return $r;
}

1;

package Net::Ajp13::Container::ModPerl::Apache;
use strict;

sub new {
    my $class = shift;
    my ($cont, $req, $res, $mh) = @_;

    bless {
        _container => $cont,
        _request => $req,
        _response => $res,
        _mhandler => $mh,
        connection => Net::Ajp13::Container::ModPerl::Apache::Connection->new(
            $cont, $req, $res, $mh
        ),
    }, $class;
}


sub send_http_header {}
sub get_basic_auth_pw {}
sub note_basic_auth_failure {}
sub handler {}
sub notes {}
sub pnotes {}
sub subprocess_env {}
sub content_type {
    my $r = shift;
    $r->{_response}->content_type(shift) if @_;
    $r->{_response}->content_type;
}

sub content_encoding {}
sub content_language {}
sub status {
    my $r = shift;
    $r->{_response}->status(shift) if @_;
    $r->{_response}->status;
}

sub status_line {}
sub headers_out {}
sub header_out {}
sub err_headers_out {}
sub err_header_out {}
sub no_cache {}
sub print {
    my $r = shift;
    $r->{_response}->print(@_);
}
sub send_fd {}
sub internal_redirect {}
sub internal_redirect_handler {}
sub custom_response {}
sub send_cgi_header {}


sub soft_timeout {}
sub hard_timeout {}
sub kill_timeout {}
sub reset_timeout {}
sub post_connection {}
sub register_cleanup {}

sub log_reason {}
sub log_error {}
sub warn {}

sub unescape_url {}
sub unescape_url_info {}
sub perl_hook {}


sub main { return undef }
sub prev { return undef }
sub next { return undef }
sub last { return undef }
sub is_main { 1 }
sub is_initial_req { 1 }

sub allowed {
    my $r = shift;
    my $bitmask;
    return 1;
}


sub lookup_uri {
    my $r = shift;
    my $uri;
}

sub lookup_file {
    my $r = shift;
    my $filename = shift;
}

sub run {
    my $r = shift;
}

sub method {
    my $r = shift;
    $r->{_request}->method;
}

sub method_number {
    my $r = shift;
    $r->{_request}->method_num;
}

sub bytes_sent {
    my $r = shift;
    $r->{bytes_sent};
}

sub the_request {
    my $r = shift;
    $r->{the_request} = shift if @_;
    $r->{the_request};
}

sub proxyreq {
    my $r = shift;
    $r->{proxyreq} = shift if @_;
    $r->{proxyreq};
}

sub header_only {
    my $r = shift;
    $r->method eq 'HEAD';
}

sub protocol {
    my $r = shift;
    $r->{_request}->protocol;
}

sub hostname {
    my $r = shift;
    $r->{hostname} = shift if @_;
    $r->{hostname};
}

sub request_time {
    my $r = shift;
    $r->{request_time} = shift if @_;
    $r->{request_time};
}

sub uri {
    my $r = shift;
    $r->{_request}->uri;
}

sub filename {
    my $r = shift;
    $r->{filename} = shift if @_;
    $r->{filename};
}

sub path_info {
    my $r = shift;
    $r->{_request}->path_info;
}

sub args {
    my $r = shift;
    $r->{_request}->query_string;
}

sub headers_in {
    my $r = shift;
    $r->{headers_in} = {@_} if @_;
    %{$r->{headers_in}};
}

sub header_in {
    my $r = shift;
    my $key = shift;
    $r->{headers_in}->{$key} = shift if @_;
    $r->{headers_in}->{$key};
}

sub content {
    my $r = shift;
    $r->{_request}->content;
}

sub read {
    my $r = shift;
}

sub get_remote_host {
    my $r = shift;
    $r->{_request}->remote_host;
}

sub get_remote_logname {
    my $r = shift;
    $r->{_request}->remote_host;
}

sub connection {
    my $r = shift;
    $r->{connection} = shift if @_;
    $r->{connection};
}

sub dir_config {
    my $r = shift;
    my $key = shift;
}

sub requires {
    my $r = shift;
    $r->{requires} = shift if @_;
    $r->{requires};
}

sub auth_type {
    my $r = shift;
    $r->{_request}->auth_type;
}

sub auth_name {
    my $r = shift;
    $r->{auth_name} = shift if @_;
    $r->{auth_name};
}

sub document_root {
    my $r = shift;
    $r->{document_root} = shift if @_;
    $r->{document_root};
}

sub allow_options {
    my $r = shift;
    $r->{allow_options} = shift if @_;
    $r->{allow_options};
}

sub get_server_port {
    my $r = shift;
    $r->{_request}->server_port;
}

sub server {
    my $r = shift;
}

1;

package Net::Ajp13::Container::ModPerl::Apache::Connection;
use strict;

sub new {
    my $class = shift;
    my ($cont, $req, $res, $mh) = @_;

    bless {
        _container => $cont,
        _request => $req,
        _response => $res,
        _mhandler => $mh,
    }, $class;
}

sub remote_host {
    my $c = shift;
    $c->{remote_host} = shift if @_;
    $c->{remote_host};
}

sub remote_ip {
    my $c = shift;
    $c->{remote_ip} = shift if @_;
    $c->{remote_ip};
}

sub local_addr {
    my $c = shift;
    $c->{local_addr} = shift if @_;
    $c->{local_addr};
}

sub remote_addr {
    my $c = shift;
    $c->{remote_addr} = shift if @_;
    $c->{remote_addr};
}

sub remote_logname {
    my $c = shift;
    $c->{remote_logname} = shift if @_;
    $c->{remote_logname};
}

sub user {
    my $c = shift;
    $c->{_request}->remote_user;
}

sub auth_type {
    my $c = shift;
    $c->{_request}->auth_type;
}

sub aborted {
    my $c = shift;
    $c->{aborted} = shift if @_;
    $c->{aborted};
}

sub fileno {
    my $c = shift;
}



1;

package Apache::Constants;

use strict;
require Exporter;


our @ISA = qw(Exporter);
our $VERSION = '1.3.31';
our @EXPORT_OK = qw(
    OK
    DECLINED
    DONE
    HTTP_CONTINUE
    HTTP_SWITCHING_PROTOCOLS
    HTTP_PROCESSING
    HTTP_OK
    HTTP_CREATED
    HTTP_ACCEPTED
    HTTP_NON_AUTHORITATIVE
    HTTP_NO_CONTENT
    HTTP_RESET_CONTENT
    HTTP_PARTIAL_CONTENT
    HTTP_MULTI_STATUS
    HTTP_MULTIPLE_CHOICES
    HTTP_MOVED_PERMANENTLY
    HTTP_MOVED_TEMPORARILY
    HTTP_SEE_OTHER
    HTTP_NOT_MODIFIED
    HTTP_USE_PROXY
    HTTP_TEMPORARY_REDIRECT
    HTTP_BAD_REQUEST
    HTTP_UNAUTHORIZED
    HTTP_PAYMENT_REQUIRED
    HTTP_FORBIDDEN
    HTTP_NOT_FOUND
    HTTP_METHOD_NOT_ALLOWED
    HTTP_NOT_ACCEPTABLE
    HTTP_PROXY_AUTHENTICATION_REQUIRED
    HTTP_REQUEST_TIME_OUT
    HTTP_CONFLICT
    HTTP_GONE
    HTTP_LENGTH_REQUIRED
    HTTP_PRECONDITION_FAILED
    HTTP_REQUEST_ENTITY_TOO_LARGE
    HTTP_REQUEST_URI_TOO_LARGE
    HTTP_UNSUPPORTED_MEDIA_TYPE
    HTTP_RANGE_NOT_SATISFIABLE
    HTTP_EXPECTATION_FAILED
    HTTP_UNPROCESSABLE_ENTITY
    HTTP_LOCKED
    HTTP_FAILED_DEPENDENCY
    HTTP_INTERNAL_SERVER_ERROR
    HTTP_NOT_IMPLEMENTED
    HTTP_BAD_GATEWAY
    HTTP_SERVICE_UNAVAILABLE
    HTTP_GATEWAY_TIME_OUT
    HTTP_VERSION_NOT_SUPPORTED
    HTTP_VARIANT_ALSO_VARIES
    HTTP_INSUFFICIENT_STORAGE
    HTTP_NOT_EXTENDED
    DOCUMENT_FOLLOWS
    PARTIAL_CONTENT
    MULTIPLE_CHOICES
    MOVED
    REDIRECT
    USE_LOCAL_COPY
    BAD_REQUEST
    AUTH_REQUIRED
    FORBIDDEN
    NOT_FOUND
    METHOD_NOT_ALLOWED
    NOT_ACCEPTABLE
    LENGTH_REQUIRED
    PRECONDITION_FAILED
    SERVER_ERROR
    NOT_IMPLEMENTED
    BAD_GATEWAY
    VARIANT_ALSO_VARIES
    M_GET
    M_PUT
    M_POST
    M_DELETE
    M_CONNECT
    M_OPTIONS
    M_TRACE
    M_PATCH
    M_PROPFIND
    M_PROPPATCH
    M_MKCOL
    M_COPY
    M_MOVE
    M_LOCK
    M_UNLOCK
    M_INVALID
    METHODS
    OPT_NONE
    OPT_INDEXES
    OPT_INCLUDES
    OPT_SYM_LINKS
    OPT_EXECCGI
    OPT_UNSET
    OPT_INCNOEXEC
    OPT_SYM_OWNER
    OPT_MULTI
    OPT_ALL
    SATISFY_ALL
    SATISFY_ANY
    SATISFY_NOSPEC
    REMOTE_HOST
    REMOTE_NAME
    REMOTE_NOLOOKUP
    REMOTE_DOUBLE_REV
    MODULE_MAGIC_NUMBER
    SERVER_VERSION
    SERVER_BUILT
    DECLINE_CMD
    DIR_MAGIC_TYPE
    OR_NONE
    OR_LIMIT
    OR_OPTIONS
    OR_FILEINFO
    OR_AUTHCFG
    OR_INDEXES
    OR_UNSET
    ACCESS_CONF
    RSRC_CONF
    OR_ALL
    RAW_ARGS
    TAKE1
    TAKE2
    ITERATE
    ITERATE2
    FLAG
    NO_ARGS
    TAKE12
    TAKE3
    TAKE23
    TAKE123
    TAKE13
);

our %EXPORT_TAGS = (
    common => [qw(OK DECLINED DONE NOT_FOUND FORBIDDEN AUTH_REQUIRED SERVER_ERROR)],
    response => [qw(
        DOCUMENT_FOLLOWS
        MOVED
        REDIRECT
        USE_LOCAL_COPY
        BAD_REQUEST
        BAD_GATEWAY
        RESPONSE_CODES
        NOT_IMPLEMENTED
        CONTINUE
        NOT_AUTHORITATIVE
    )],
    methods => [qw(
        METHODS
        M_CONNECT
        M_DELETE
        M_GET
        M_INVALID
        M_OPTIONS
        M_POST
        M_PUT
        M_TRACE 
        M_PATCH
        M_PROPFIND
        M_PROPPATCH
        M_MKCOL
        M_COPY
        M_MOVE
        M_LOCK
        M_UNLOCK
    )],
    options => [qw(
        OPT_NONE
        OPT_INDEXES
        OPT_INCLUDES 
        OPT_SYM_LINKS
        OPT_EXECCGI
        OPT_UNSET
        OPT_INCNOEXEC
        OPT_SYM_OWNER
        OPT_MULTI
        OPT_ALL
    )],
    satisfy => [qw(SATISFY_ALL SATISFY_ANY SATISFY_NOSPEC)],
    remotehost => [qw(REMOTE_HOST REMOTE_NAME REMOTE_NOLOOKUP REMOTE_DOUBLE_REV)],
    http => [qw(
        HTTP_OK
        HTTP_MOVED_TEMPORARILY
        HTTP_MOVED_PERMANENTLY
        HTTP_METHOD_NOT_ALLOWED 
        HTTP_NOT_MODIFIED
        HTTP_UNAUTHORIZED
        HTTP_FORBIDDEN
        HTTP_NOT_FOUND
        HTTP_BAD_REQUEST
        HTTP_INTERNAL_SERVER_ERROR
        HTTP_NOT_ACCEPTABLE 
        HTTP_NO_CONTENT
        HTTP_PRECONDITION_FAILED
        HTTP_SERVICE_UNAVAILABLE
        HTTP_VARIANT_ALSO_VARIES
    )],
    server => [qw(MODULE_MAGIC_NUMBER SERVER_VERSION SERVER_BUILT)],
    config => [qw(DECLINE_CMD)],
    types => [qw(DIR_MAGIC_TYPE)],
    override => [qw(
        OR_NONE
        OR_LIMIT
        OR_OPTIONS
        OR_FILEINFO
        OR_AUTHCFG
        OR_INDEXES
        OR_UNSET
        OR_ALL
        ACCESS_CONF
        RSRC_CONF
    )],
    args_how => [qw(
        RAW_ARGS
        TAKE1
        TAKE2
        TAKE12
        TAKE3
        TAKE23
        TAKE123
        ITERATE
        ITERATE2
        FLAG
        NO_ARGS
    )],
);

sub OK { 0 }
sub DECLINED { -1 }
sub DONE { -2 }

sub HTTP_CONTINUE                     { 100 }
sub HTTP_SWITCHING_PROTOCOLS          { 101 }
sub HTTP_PROCESSING                   { 102 }
sub HTTP_OK                           { 200 }
sub HTTP_CREATED                      { 201 }
sub HTTP_ACCEPTED                     { 202 }
sub HTTP_NON_AUTHORITATIVE            { 203 }
sub HTTP_NO_CONTENT                   { 204 }
sub HTTP_RESET_CONTENT                { 205 }
sub HTTP_PARTIAL_CONTENT              { 206 }
sub HTTP_MULTI_STATUS                 { 207 }
sub HTTP_MULTIPLE_CHOICES             { 300 }
sub HTTP_MOVED_PERMANENTLY            { 301 }
sub HTTP_MOVED_TEMPORARILY            { 302 }
sub HTTP_SEE_OTHER                    { 303 }
sub HTTP_NOT_MODIFIED                 { 304 }
sub HTTP_USE_PROXY                    { 305 }
sub HTTP_TEMPORARY_REDIRECT           { 307 }
sub HTTP_BAD_REQUEST                  { 400 }
sub HTTP_UNAUTHORIZED                 { 401 }
sub HTTP_PAYMENT_REQUIRED             { 402 }
sub HTTP_FORBIDDEN                    { 403 }
sub HTTP_NOT_FOUND                    { 404 }
sub HTTP_METHOD_NOT_ALLOWED           { 405 }
sub HTTP_NOT_ACCEPTABLE               { 406 }
sub HTTP_PROXY_AUTHENTICATION_REQUIRED{ 407 }
sub HTTP_REQUEST_TIME_OUT             { 408 }
sub HTTP_CONFLICT                     { 409 }
sub HTTP_GONE                         { 410 }
sub HTTP_LENGTH_REQUIRED              { 411 }
sub HTTP_PRECONDITION_FAILED          { 412 }
sub HTTP_REQUEST_ENTITY_TOO_LARGE     { 413 }
sub HTTP_REQUEST_URI_TOO_LARGE        { 414 }
sub HTTP_UNSUPPORTED_MEDIA_TYPE       { 415 }
sub HTTP_RANGE_NOT_SATISFIABLE        { 416 }
sub HTTP_EXPECTATION_FAILED           { 417 }
sub HTTP_UNPROCESSABLE_ENTITY         { 422 }
sub HTTP_LOCKED                       { 423 }
sub HTTP_FAILED_DEPENDENCY            { 424 }
sub HTTP_INTERNAL_SERVER_ERROR        { 500 }
sub HTTP_NOT_IMPLEMENTED              { 501 }
sub HTTP_BAD_GATEWAY                  { 502 }
sub HTTP_SERVICE_UNAVAILABLE          { 503 }
sub HTTP_GATEWAY_TIME_OUT             { 504 }
sub HTTP_VERSION_NOT_SUPPORTED        { 505 }
sub HTTP_VARIANT_ALSO_VARIES          { 506 }
sub HTTP_INSUFFICIENT_STORAGE         { 507 }
sub HTTP_NOT_EXTENDED                 { 510 }

sub DOCUMENT_FOLLOWS    { 200 }
sub PARTIAL_CONTENT     { 206 }
sub MULTIPLE_CHOICES    { 300 }
sub MOVED               { 301 }
sub REDIRECT            { 302 }
sub USE_LOCAL_COPY      { 304 }
sub BAD_REQUEST         { 400 }
sub AUTH_REQUIRED       { 401 }
sub FORBIDDEN           { 403 }
sub NOT_FOUND           { 404 }
sub METHOD_NOT_ALLOWED  { 405 }
sub NOT_ACCEPTABLE      { 406 }
sub LENGTH_REQUIRED     { 411 }
sub PRECONDITION_FAILED { 412 }
sub SERVER_ERROR        { 500 }
sub NOT_IMPLEMENTED     { 501 }
sub BAD_GATEWAY         { 502 }
sub VARIANT_ALSO_VARIES { 506 }


sub M_GET       {  0 }
sub M_PUT       {  1 }
sub M_POST      {  2 }
sub M_DELETE    {  3 }
sub M_CONNECT   {  4 }
sub M_OPTIONS   {  5 }
sub M_TRACE     {  6 }
sub M_PATCH     {  7 }
sub M_PROPFIND  {  8 }
sub M_PROPPATCH {  9 }
sub M_MKCOL     { 10 }
sub M_COPY      { 11 }
sub M_MOVE      { 12 }
sub M_LOCK      { 13 }
sub M_UNLOCK    { 14 }
sub M_INVALID   { 15 }

sub METHODS     { 16 }

sub OPT_NONE      { 0 }
sub OPT_INDEXES   { 1 }
sub OPT_INCLUDES  { 2 }
sub OPT_SYM_LINKS { 4 }
sub OPT_EXECCGI   { 8 }
sub OPT_UNSET     { 16 }
sub OPT_INCNOEXEC { 32 }
sub OPT_SYM_OWNER { 64 }
sub OPT_MULTI     { 128 }
sub OPT_ALL { 1|2|4|8 }  # OPT_INDEXES|OPT_INCLUDES|OPT_SYM_LINKS|OPT_EXECCGI

sub SATISFY_ALL    { 0 }
sub SATISFY_ANY    { 1 }
sub SATISFY_NOSPEC { 2 }

sub REMOTE_HOST { 0 }
sub REMOTE_NAME { 1 }
sub REMOTE_NOLOOKUP { 2 }
sub REMOTE_DOUBLE_REV { 3 }

sub MODULE_MAGIC_NUMBER { 19990320 }
sub SERVER_VERSION { "Atami/$VERSION" }
sub SERVER_BUILT { 'Feb  4 2004 10:31:58' }

sub DECLINE_CMD { "\a\b" }

sub DIR_MAGIC_TYPE { 'httpd/unix-directory' }


sub OR_NONE     {   0 }
sub OR_LIMIT    {   1 }
sub OR_OPTIONS  {   2 }
sub OR_FILEINFO {   4 }
sub OR_AUTHCFG  {   8 }
sub OR_INDEXES  {  16 }
sub OR_UNSET    {  32 }
sub ACCESS_CONF {  64 }
sub RSRC_CONF   { 128 }
sub OR_ALL { 1|2|4|8|16 }

sub RAW_ARGS {  0 }
sub TAKE1    {  1 }
sub TAKE2    {  2 }
sub ITERATE  {  3 }
sub ITERATE2 {  4 }
sub FLAG     {  5 }
sub NO_ARGS  {  6 }
sub TAKE12   {  7 }
sub TAKE3    {  8 }
sub TAKE23   {  9 }
sub TAKE123  { 10 }
sub TAKE13   { 11 }

1;


__END__
