package Net::Ajp13::Log;
use strict;

my %LEVEL = (
    emerg   => 0,
    alert   => 1,
    crit    => 2,
    err     => 3, error => 3,
    warning => 4,
    notice  => 5,
    info    => 6,
    debug   => 7,
);

sub new {
    my $class = shift;
    my $fh = shift;
    my $level = shift;
    my $fmt = shift;

    select $fh; $|=1; select STDOUT;
    my $self =  bless {
        fh => $fh,
        level => $LEVEL{info},
        fmt => $fmt || sub {
            my $level = shift;
            my $str = shift; 
            sprintf "[%s] [%s] %s\n",
                scalar localtime, $level, $str;
        },
    }, $class;
    $self->level($level || 'info');
    return $self;
}

sub level {
    my $self = shift;
    if (@_) {
        my $l = shift;
        unless (exists $LEVEL{$l}) {
            $self->printf('warning', 'undefined Log level: "%s"', $l);
            $l = 'warning';
        }
        $self->{level} = $LEVEL{$l};
    }
    $self->{level};
}

sub print {
    my $self = shift;
    my $level = shift;
    if (!exists $LEVEL{$level}) {
        $self->printf('warning', 'undefined Log level: "%s"', $level);
        $level = 'warning';
    }

    return if $self->level < $LEVEL{$level};

    my $fh = $self->{fh};
    print $fh $self->{fmt}->($level, join $,, @_);
}

sub printf {
    my $self = shift;
    my $level = shift;
    my $fmt = shift;

    $self->print($level, sprintf $fmt, @_);
}


1;
__END__
