package Net::Ajp13::Packet;
our $VERSION = '0.0.1';

use strict;
use constant AJP13_WS_HEADER   => 0x1234;
use constant AJP13_SW_HEADER   => 0x4142;


sub new {
    my $class = shift;
    my $source = shift;
    my $len = shift;

    bless {
        source => $source || '',
        src_len => $len || 0,
        magic => '',
        'length' => 0,
        pos => 0,
    }, $class;
}

sub recycle {
    my $self = shift;
    $self->{source} = '';
    $self->{src_len} = 0;
    $self->{magic} = '';
    $self->{'length'} = 0;
    $self->{pos} = 0;
}

sub append {
    my $self = shift;
    $self->{source} .= shift;
    $self->{src_len} += shift;
}

sub magic {
    my $self = shift;
    if (@_) {
        $self->{magic} = shift;
    }
    $self->{magic};
}

sub length {
    my $self = shift;
    if (@_) {
        $self->{'length'} = shift;
    }
    $self->{'length'};
}

sub is_valid {
    my $self = shift;
    $self->magic($self->get_int());
    return undef if $self->magic != AJP13_WS_HEADER;
    $self->length($self->get_int());

    return $self;
}

sub get_int {
#    my $self = shift;
    my $int = unpack 'n', substr $_[0]->{source}, $_[0]->{pos}, 2;
    $_[0]->{pos} += 2;
    return $int;
}

sub peek_int {
    my $self = shift;
    my $int = unpack 'n', substr $self->{source}, $self->{pos}, 2;
    return $int;
}

sub get_byte {
    my $self = shift;
    my $byte = unpack 'C', substr $self->{source}, $self->{pos}, 1;
    $self->{pos} += 1;
    return $byte;
}

sub get_bool {
    my $self = shift;
    my $bool = $self->get_byte(@_);
    return $bool == 1 ? 1 : undef;
}

sub get_message {
#    my $self = shift;

    #my $l = $self->get_int();
    my $l = unpack 'n', substr $_[0]->{source}, $_[0]->{pos}, 2;
    $_[0]->{pos} += 2;
    if ($l == 0xFFFF || $l == -1) {
        return undef;
    }
    my $msg = substr $_[0]->{source}, $_[0]->{pos}, $l;
    $_[0]->{pos} += $l;
    $_[0]->{pos} += 1; # '\0'
    return $msg;    
}


sub set_int {
    my $self = shift;
    substr($self->{source}, $_[1], 2) = pack 'n', $_[0];

    return 2;
}

sub append_byte {
    my $self = shift;
    $self->{source} .= pack 'C', $_[0];
    $self->{'length'} += 1;
}

sub append_int {
    #my $self = shift;
    $_[0]->{source} .= pack 'n', $_[1];
    $_[0]->{'length'} += 2;
}

sub append_bool {
    my $self = shift;
    if ($_[0]) {
        $self->{source} .= pack 'C', 1;
    }
    else {
        $self->{source} .= pack 'C', 0;
    }
    $self->{'length'} += 1;
}

sub append_message {
    my $self = shift;

    if ($_[0] eq '') {
        $self->append_int(0);
        $self->append_byte(0);
        return $self->{'length'} += 3;
    }

    $self->append_int(CORE::length($_[0]));
    $self->{source} .= $_[0];
    $self->{'length'} += CORE::length($_[0]);
    $self->append_byte(0);
}

sub dump {
    my $self = shift;
    my $packet = $self->{source};
    my ($method_name) = (caller(1))[3];
    printf "%s():\n%s\n", $method_name,
           join ' ', map { sprintf "%02x", ord $_ } split //, $packet;
    printf "%s():\n%s\n", $method_name,
           join '  ', map { m/[\d \w\._]/ ? $_ : '.' } split //, $packet;
    printf "-- %s --\n", scalar localtime;
}

1;
__END__
