package Net::Ajp13::Request;

use strict;

use constant METHOD_NUM => [qw(
    OPTIONS GET HEAD POST PUT DELETE TRACE PROPFIND PROPPATCH MKCOL
    COPY MOVE LOCK UNLOCK ACL REPORT VERSION-CONTROL CHECKIN CHECKOUT UNCHECKOUT
    SEARCH MKWORKSPACE UPDATE LABEL MERGE BASELINE-CONTROL MKACTIVITY
)];
use constant A2E => [
   0,  1,  2,  3, 55, 45, 46, 47, 22,  5, 21, 11, 12, 13, 14, 15,
  16, 17, 18, 19, 60, 61, 50, 38, 24, 25, 63, 39, 28, 29, 30, 31,
  64, 90,127,123, 91,108, 80,125, 77, 93, 92, 78,107, 96, 75, 97,
 240,241,242,243,244,245,246,247,248,249,122, 94, 76,126,110,111,
 124,193,194,195,196,197,198,199,200,201,209,210,211,212,213,214,
 215,216,217,226,227,228,229,230,231,232,233,173,224,189, 95,109,
 121,129,130,131,132,133,134,135,136,137,145,146,147,148,149,150,
 151,152,153,162,163,164,165,166,167,168,169,192, 79,208,161,  7,
  32, 33, 34, 35, 36, 37,  6, 23, 40, 41, 42, 43, 44,  9, 10, 27,
  48, 49, 26, 51, 52, 53, 54,  8, 56, 57, 58, 59,  4, 20, 62,255,
  65,170, 74,177,159,178,106,181,187,180,154,138,176,202,175,188,
 144,143,234,250,190,160,182,179,157,218,155,139,183,184,185,171,
 100,101, 98,102, 99,103,158,104,116,113,114,115,120,117,118,119,
 172,105,237,238,235,239,236,191,128,253,254,251,252,186,174, 89,
  68, 69, 66, 70, 67, 71,156, 72, 84, 81, 82, 83, 88, 85, 86, 87,
 140, 73,205,206,203,207,204,225,112,221,222,219,220,141,142,223
];


sub new {
    my $class = shift;
    bless {
        status => 200,
        method_num => undef,
        protocol => undef,
        uri => undef,
        remote_addr => undef,
        remote_host => undef,
        server_name => undef,
        server_port => undef,
        scheme => 'http',
        content_length => -1,
        content => undef,
        content_type => undef,
        authorization => undef,
        remote_user => undef,
        auth_type => undef,
        query_string => undef,
        jvm_route => undef,
        ssl => 0,
        secret => undef,
        header => {},
        attribute => {},
        parameter => {},
        parameter_parsed => 0,
        session => undef,
    }, $class;
}

sub recycle {
    my $self = shift;
    $self->{status} = 200;
    $self->{method_num} = undef;
    $self->{protocol} = undef;
    $self->{uri} = undef;
    $self->{remote_addr} = undef;
    $self->{remote_host} = undef;
    $self->{server_name} = undef;
    $self->{server_port} = undef;
    $self->{scheme} = 'http';
    $self->{content_length} = -1;
    $self->{content} = undef;
    $self->{content_type} = undef;
    $self->{authorization} = undef;
    $self->{remote_user} = shift;
    $self->{auth_type} = shift;
    $self->{jvm_route} = shift;
    $self->{ssl} = 0;
    $self->{secret} = undef;
    $self->{header} = {};
    $self->{attribute} = {};
    $self->{parameter} = {};
    $self->{parameter_parsed} = 0;
    $self->{session} = undef;
    return $self; 
}

sub status {
    my $self = shift;
    $self->{status} = shift if @_;
    $self->{status};
}

sub method {
    my $self = shift;
    METHOD_NUM->[$self->{method_num} - 1];
}

sub method_num {
    my $self = shift;
    $self->{method_num} = shift if @_;
    $self->{method_num};
}

sub protocol {
    my $self = shift;
    $self->{protocol} = shift if @_;
    $self->{protocol};
}

sub uri {
    my $self = shift;
    $self->{uri} = shift if @_;
    $self->{uri};
}

sub remote_addr {
    my $self = shift;
    $self->{remote_addr} = shift if @_;
    $self->{remote_addr};
}

sub remote_host {
    my $self = shift;
    $self->{remote_host} = shift if @_;
    $self->{remote_host};
}

sub server_name {
    my $self = shift;
    $self->{server_name} = shift if @_;
    $self->{server_name};
}

sub server_port {
    my $self = shift;
    $self->{server_port} = shift if @_;
    $self->{server_port};
}

sub scheme {
    my $self = shift;
    $self->{scheme} = shift if @_;
    $self->{scheme};
}

sub remote_user {
    my $self = shift;
    $self->{remote_user} = shift if @_;
    $self->{remote_user};
}

sub auth_type {
    my $self = shift;
    $self->{auth_type} = shift if @_;
    $self->{auth_type};
}

sub query_string {
    my $self = shift;
    $self->{query_string} = shift if @_;
    $self->{query_string};
}

sub path_info {
    my $self = shift;
    return undef;
}

sub jvm_route {
    my $self = shift;
    $self->{jvm_route} = shift if @_;
    $self->{jvm_route};
}

sub ssl {
    my $self = shift;
    $self->{ssl} = shift if @_;
    $self->{ssl};
}

sub secure {
    my $self = shift;
    $self->{securel} = shift if @_;
    $self->{secure};
}

sub secret {
    my $self = shift;
    $self->{secret} = shift if @_;
    $self->{secret};
}

sub header {
    my $self = shift;
    my $name = lc shift;
    if (@_) {
        $self->{header}->{$name} = join '', @_;
    } 
    $self->{header}->{$name};
}

sub attribute {
    my $self = shift;
    my $name = lc shift;
    if (@_) {
        $self->{attribute}->{$name} = join '', @_;
    } 
    $self->{attribute}->{$name};
}

sub session {
    my $self = shift;
    $self->{session} = shift if @_;
    $self->{session};
}

sub authorization {
    my $self = shift;
    if (@_) {
        $self->{authorization} = join '', @_;
    } 
    $self->{authorization};
}

sub content {
    my $self = shift;
    if (@_) {
        $self->{content} = join '', @_;
    } 
    $self->{content};
}

sub append_content {
    my $self = shift;
    $self->{content} .= join '', @_;
}

sub content_length {
    my $self = shift;
    if (@_) {
        $self->{content_length} = join '', @_;
    } 
    $self->{content_length};
}

sub content_type {
    my $self = shift;
    if (@_) {
        $self->{content_type} = join '', @_;
    } 
    $self->{content_type};
}

sub get_parameter {
    my $self = shift;
    my $key = shift;

    return undef if !$self->query_string && !$self->content;

    if ($self->{parameter_parsed} != 1) {
        $self->_parse_params();
        $self->{parameter_parsed} = 1;
    }
    return undef unless exists $self->{parameter}->{$key};
    return wantarray ? @{$self->{parameter}->{$key}} : $self->{parameter}->{$key}->[0];
}

sub _parse_params {
    my $self = shift;

    my $src = $self->query_string;
    if ($self->method eq 'POST' && $self->content_length > 0) {
        $src .= $self->content;
    }

    my @pairs = split /[&;]/, $src;
    my ($key, $value);
    for (@pairs) {
        ($key, $value) = split '=', $_, 2;
        next unless defined $key;
        $value = '' unless defined $value;
        $key = _unescape($key);
        $value = _unescape($value);
        $self->{parameter}->{$key} = [] unless exists $self->{parameter}->{$key};
        push @{$self->{parameter}->{$key}}, $value;
    }
}

sub _unescape {
    my $src = shift;
    return undef unless defined($src);

    $src =~ tr/+/ /;
    my $is_EBCDIC = "\t" ne "\011";
    if ($is_EBCDIC) {
        $src =~ s/%([0-9a-fA-F]{2})/chr A2E->[hex($1)]/ge;
    }
    else {
        $src =~ s/%(?:([0-9a-fA-F]{2})|u([0-9a-fA-F]{4}))/
            defined($1) ? chr hex($1) : _decode_utf8(hex($2))/ge;
    }
    return $src;
}

sub _decode_utf8 {
    my $src = shift;
    if ($src < 0x80) {
        return sprintf("%c", $src);
    }
    elsif ($src < 0x800) {
        return sprintf("%c%c", 0xc0 | ($src >> 6), 0x80 | ($src & 0x3f));
    }
    elsif ($src < 0x10000) {
        return sprintf("%c%c%c",
            0xe0 | ($src >> 12),
            0x80 | (($src >> 6) & 0x3f),
            0x80 | ($src & 0x3f));
    }
    elsif ($src < 0x200000) {
        return sprintf("%c%c%c%c",
            0xf0 | ($src >> 18),
            0x80 | (($src >> 12) & 0x3f),
            0x80 | (($src >> 6) & 0x3f),
            0x80 | ($src & 0x3f));
    }
    elsif ($src < 0x4000000) {
        return sprintf("%c%c%c%c%c",
            0xf8 | ($src >> 24),
            0x80 | (($src >> 18) & 0x3f),
            0x80 | (($src >> 12) & 0x3f),
            0x80 | (($src >> 6) & 0x3f),
            0x80 | ($src & 0x3f));
    }
    elsif ($src < 0x80000000) {
        return sprintf("%c%c%c%c%c%c",
            0xfe | ($src >> 30),
            0x80 | (($src >> 24) & 0x3f),
            0x80 | (($src >> 18) & 0x3f),
            0x80 | (($src >> 12) & 0x3f),
            0x80 | (($src >> 6)  & 0x3f),
            0x80 | ($src & 0x3f));
    }
    else {
        return '?'; # utf8(0xfffd)?
    }
} 

1;
__END__
