package Net::Ajp13::Response;

use strict;
use constant HTTP_STATUS_LINE => {
    100 => "100 Continue",
    101 => "101 Switching Protocols",
    102 => "102 Processing",
    200 => "200 OK",
    201 => "201 Created",
    202 => "202 Accepted",
    203 => "203 Non-Authoritative Information",
    204 => "204 No Content",
    205 => "205 Reset Content",
    206 => "206 Partial Content",
    206 => "207 Multi-Status",
    300 => "300 Multiple Choices",
    301 => "301 Moved Permanently",
    302 => "302 Found",
    303 => "303 See Other",
    304 => "304 Not Modified",
    305 => "305 Use Proxy",
    306 => "306 unused",
    307 => "307 Temporary Redirect",
    400 => "400 Bad Request",
    401 => "401 Authorization Required",
    402 => "402 Payment Required",
    403 => "403 Forbidden",
    404 => "404 Not Found",
    405 => "405 Method Not Allowed",
    406 => "406 Not Acceptable",
    407 => "407 Proxy Authentication Required",
    408 => "408 Request Time-out",
    409 => "409 Conflict",
    410 => "410 Gone",
    411 => "411 Length Required",
    412 => "412 Precondition Failed",
    413 => "413 Request Entity Too Large",
    414 => "414 Request-URI Too Large",
    415 => "415 Unsupported Media Type",
    416 => "416 Requested Range Not Satisfiable",
    417 => "417 Expectation Failed",
    422 => "422 Unprocessable Entity",
    423 => "423 Locked",
    424 => "424 Failed Dependency",
    500 => "500 Internal Server Error",
    501 => "501 Method Not Implemented",
    502 => "502 Bad Gateway",
    503 => "503 Service Temporarily Unavailable",
    504 => "504 Gateway Time-out",
    505 => "505 HTTP Version Not Supported",
    506 => "506 Variant Also Negotiates",
    507 => "507 Insufficient Storage",
    510 => "510 Not Extended"
};



sub new {
    my $class = shift;
    bless {
        status => 200,
        content_type => 'text/plain',
        content_length =>  -1,
        content => '',
        header => {},
        wire_protocol_handler => undef,
        closed => 0,
        require_send_header => 1,
        bytes_sent => 0,
    }, $class;
}

sub closed {
    my $self = shift;
    $self->{closed} = shift if @_;
    $self->{closed};
}

sub require_send_header {
    my $self = shift;
    $self->{require_send_header} = shift if @_;
    $self->{require_send_header};

}


sub wire_handler {
    my $self = shift;
    $self->{wire_protocol_handler} = shift if @_;
    $self->{wire_protocol_handler};
}

sub recycle {
    my $self = shift;

    $self->close();
    $self->{status} = 200;
    $self->{content_type} = 'text/plain',
    $self->{content_length} = -1;
    $self->{content} = '';
    $self->{header} = {};
    $self->{wire_protocol_handler} = undef;
    $self->{closed} = 0;
    $self->{require_send_header} = 1;
    $self->{bytes_sent} = 0;
    $self;
}

sub status {
    my $self = shift;
    $self->{status} = shift if @_;
    $self->{status};
}

sub status_message {
    my $self = shift;
    HTTP_STATUS_LINE->{$self->{status}};
}

sub content_type {
    my $self = shift;
    $self->{content_type} = shift if @_;
    $self->{content_type};
}

sub content_length {
    my $self = shift;
    if (@_) {
        my $l = shift;
        $self->{content_length} = $l;
        $self->header('Content-Length', $l);
    }
    $self->{content_length};
}

sub content {
    my $self = shift;
    $self->{content} = join '', @_ if @_;
    $self->{content};
}

sub header {
    my $self = shift;
    my $name = shift;
    if (@_) {
        $self->{header}->{$name} = join '', @_;
    }
    $self->{header}->{$name};
}

sub headers {
    my $self = shift;
    return keys %{$self->{header}};
}

sub send_redirect {
    my $self = shift;
    my $url = shift;
    return if $self->closed;

    $self->header('Location', $url);
    $self->status(303);
}

sub send_header {
    my $self = shift;
    return if $self->closed;
    return unless $self->require_send_header;

    $self->content_length(length $self->content);
    $self->wire_handler->send_header($self);
    $self->require_send_header(0);
}

sub print {
    my $self = shift;
    $self->{content} .= join '', @_;
}

sub printf {
    my $self = shift;
    my $fmt = shift;
    $self->{content} .= sprintf $fmt, @_;
}

sub flush {
    my $self = shift;
    return if $self->closed;

    $self->send_header();
    $self->wire_handler->send_body_chunk($self);
    $self->content(undef);
}

sub close {
    my $self = shift;
    return if $self->closed;

    $self->send_header();
    $self->wire_handler->send_body_chunk($self) if $self->content;
    $self->wire_handler->send_end_body_chunk();
    
    $self->closed(1);
}

1;
__END__
