package Net::Ajp13::SessionManager;

use Net::Ajp13::Session;
use strict;
use threads;
use threads::shared;

sub new {
    my $class = shift;
    my $conf = shift;

    my $self = bless {
        conf => $conf,
        attribute => &share({}),
        _seq => 0,
    }, $class;
    $self->init();

    return $self;
}

sub init {
    my $self = shift;
    printf "SessionManager::init(%s)\n", $self;

}

sub destroy {
    my $self = shift;
    printf "SessionManager::destroy(%s)\n", $self;

}


sub attribute {
    my $self = shift;
    my $id = shift;
    my $key = shift;

    lock($self->{attribute});
    if (@_) {
        if (!exists $self->{attribute}->{$id}) {
            $self->{attribute}->{$id} = &share({});
        }
        $self->{attribute}->{$id}->{$key} = shift;
    }

    return undef if !exists $self->{attribute}->{$id};
    $self->{attribute}->{$id}->{$key};
}

sub destroy_by_id {
    my $self = shift;
    my $id = shift;

    return unless exists $self->{attribute}->{$id};

    lock($self->{attribute});
    for my $key (keys %{$self->{attribute}->{$id}}) {
        delete $self->{attribute}->{$id}->{$key};
    }
    delete $self->{attribute}->{$id};
}


sub get_session {
    my $self = shift;
    my $req = shift;
    my $res = shift;

    my $id = $self->get_session_id($req);
    $self->set_session_id($res, $id);

    return Net::Ajp13::Session->new($self, $id);
}

sub get_session_id {
    my $self = shift;
    my $req = shift;

    my $raw_cookie = $req->header('Cookie');
    if (!$raw_cookie) {
        return $self->create_session_id($req);
    }

    for (split '; ?', $raw_cookie) {
        s/\s*(.*?)\s*/$1/;
        my ($key, $value) = split '=', $_, 2;
        next if $key ne 'atami_session';

        return $value;
    }

    return $self->create_session_id($req);
}

sub create_session_id {
    my $self = shift;
    my $req = shift;
    $self->{_seq}++;

    return sprintf "%d:%d:%d:%d", time(), $$, threads->tid, $self->{_seq};
}

sub set_session_id {
    my $self = shift;
    my $res = shift;
    my $id = shift;

    my $raw_cookie = sprintf '%s=%s;', 'atami_session', $id;
    $res->header('Set-Cookie', $raw_cookie);
}


1;


1;
__END__
